<?php

require 'vendor/autoload.php'; // Load Google Client Library

use Google\Auth\CredentialsLoader;
use Google\Auth\OAuth2;
use GuzzleHttp\Client;

function generateJwt($credentials) {
    $privateKey = $credentials['private_key'];
    $clientEmail = $credentials['client_email'];
    
    $header = json_encode(['alg' => 'RS256', 'typ' => 'JWT']);
    $payload = json_encode([
        'iss' => $clientEmail,
        'scope' => 'https://www.googleapis.com/auth/firebase.messaging',
        'aud' => 'https://oauth2.googleapis.com/token',
        'exp' => time() + 3600, // Token expiration time
        'iat' => time()
    ]);

    $base64Header = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode($header));
    $base64Payload = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode($payload));
    
    $signature = '';
    openssl_sign($base64Header . '.' . $base64Payload, $signature, $privateKey, OPENSSL_ALGO_SHA256);
    
    $base64Signature = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode($signature));
    
    return $base64Header . '.' . $base64Payload . '.' . $base64Signature;
}

function fetchAccessToken($serviceAccountPath) {
    $credentials = json_decode(file_get_contents($serviceAccountPath), true);
    var_dump($credentials);
    $client = new Client();
    
    // Prepare the request
    $response = $client->post('https://oauth2.googleapis.com/token', [
        'form_params' => [
            'grant_type' => 'urn:ietf:params:oauth:grant-type:jwt-bearer',
            'assertion' => generateJwt($credentials) // Generate JWT
        ],
        'verify' => false // Ensure SSL verification is enabled
    ]);

    return json_decode($response->getBody(), true);
}

function sendPushNotification($token, $title, $body, $serviceAccountPath) {
    // Fetch OAuth2 token
    $auth = fetchAccessToken($serviceAccountPath);
    if (isset($auth['error'])) {
        return 'Error fetching OAuth2 token: ' . $auth['error_description'];
    }
    
    $accessToken = $auth['access_token'];
     $data = [
                "appName" => "dcf-sfa",
                "token" => $accessToken
            ];

            // Convert the PHP array to a JSON string
            $jsonData = json_encode($data);

            // Output or use $jsonData as needed
            echo $jsonData;die;
    // Prepare FCM API URL
    $fcmUrl = 'https://fcm.googleapis.com/v1/projects/dcf-sfa/messages:send';
    
    // Prepare the notification payload
    $notification = [
        'message' => [
            'token' => $token, // Device token
            'notification' => [
                'title' => $title,
                'body' => $body,
            ],
        ],
    ];

    // Set the headers including the generated OAuth2 token
    $headers = [
        'Authorization: Bearer ' . $accessToken,
        'Content-Type: application/json',
    ];

    // Initialize cURL to make the request
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $fcmUrl);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Enable SSL verification
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($notification));

    // Execute the request
    $result = curl_exec($ch);
    
    // Check for SSL certificate errors or other issues
    if ($result === false) {
        $error = curl_error($ch);
        curl_close($ch);
        return 'cURL error: ' . $error;
    }
    
    curl_close($ch);

    // Return the result
    return $result;
}

// Example usage
$deviceToken = 'etfFWhfoQKaGeKfvQwAK-b:APA91bGVn3DVdLEY8nlJmcVlfmoSWFJf5YkvAeZN6sM6UAoLhnhwraKSBIDoJ5BFJ_Jfw_nxDWo1Ovz53M0WzMc3aE4K1B6bbIYuXNkBewiL1Xu0wPo5E_s'; // Token from the device
$title = 'Hello!';
$body = 'This is a test notification.';
$serviceAccountPath = 'https://localhost:8081/sfa/fcm/serviceAccountKey.json'; // Path to your service account JSON file

$response = sendPushNotification($deviceToken, $title, $body, $serviceAccountPath);

echo $response;
